; **********************************************************
; **********************************************************
; ****              ****
; ****  ASCII-Daten auf Nokia 3310 Display darstellen  ****
; ****              ****
; **********************************************************
; **********************************************************

;  ****   X und Y Adresse im Display aus XPosC und YPos   ****
;  ****  berechnen und setzen        ****
;  ****  !!! 0<=XPosC<=13, 0<=YPos<=5 !!!    ****

LCD_SetCursor  bcf  STATUS,C
    rlf  XPosC,w    ; in w jetzt 2* XPosC
    movwf  XPosD
    rlf  XPosD,f    ; in XPosD jetzt 4*XPosC
    addwf  XPosD,w    ; in w jetzt 6*XPosC

;    movwf  XPosD    ; wird nicht weiter bentigt
          ; da der Adresscouter im Display incrementiert wird

    iorlw  10000000b  ; LCD-Command set X adr bilden
    movwf  DispBuff

    movf  YPos,w
    iorlw  01000000b  ; LCD-Command set y adr bilden
    movwf  DispBuff+1

    movlw  2    ; 2Byte zu bertragen
    call  LCD_ComSend

    return

;  ****  aus XPosC und YPos Adresse im Bildschirmspeicher bilden  ****
;  ****  Rckgabe der Adresse in w        ****

LCD_GetSCRADR  bcf  STATUS,C
    rlf  YPos,w
    movwf  ADRL
    rlf  ADRL,f    ; in ADRL jetzt 4*YPos
    rlf  ADRL,f    ; in ADRL jetzt 8*YPos
    rlf  ADRL,f    ; in ADRL jetzt 16*YPos

    subwf  ADRL,w    ; in w jetzt 14*YPos

    addwf  XPosC,w    ; in w jetzt YPos*14+XPosC
    addlw  ScreenBuff
    return


;  ****  _ShowCursor=0
;  ****  ===============
;  ****  String aus RAM auf Display ausgeben      ****
;  ****   String beginnt bei FSR          ****
;  ****  Anzahl Zeichen steht in w        ****
;  ****  Cursor muss vorher gesetzt sein        ****
;  ****  Kursor wird um w Zeichen verschoben      ****

;  ****  _ShowCursor=1
;  ****  ===============
;  ****  Es wird nur ein Zeichen, welches ab FSR im RAM steht    ****
;  ****  normal(_CursorOn=0) oder als XOR mit CursorMask(_CursorOn=1)  ****
;  ****  ausgegeben.              ****

LCD_PrintStr  movwf  BZ3    ; Anzahl darzustellender Zeichen

NextChar  movf  0,w    ; aktuelles ASCII-Zeichen in w
    movwf  ASCIICode

    movf  FSR,w
    movwf  MerkFSR    ; FSR Zwischenspeichern

  ; Zeichen auf Display darstellen

    clrf  ASCIIOffset  ; Zhler fr 5 Byte (=1 Zeichen)
    movlw  DispBuff
    movwf  FSR

NextASCByte  clrwdt
    call  GetASCIIByte  ; aktuelles Bitmuster holen
    btfsc  _CursorOn  ; Kursor einblenden?
    xorwf  CursorMask,w
    movwf  0    ; in DisplayBuffer
    incf  FSR,f
    incf  ASCIIOffset,f  ; Offset fr aktuelles Bitmuster
    movlw  5
    subwf  ASCIIOffset,w
    btfss  STATUS,Z  ; 5 Byte gesendet?
    goto  NextASCByte

    clrf  0    ; ein Byte frei zwischen zwei Zeichen
    movlw  6
    call  LCD_DataSend  ; Zeichen darstellen

    btfss  _ShowCursor
    goto  WSCR

    call  LCD_SetCursor  ; Cursorposition (Adresse) im Display wieder
          ; herstellen, da Adresse sich durch Einblenden
          ; des Cursors verschiebt

    return      ; Ende, wenn _ShowCursor=1

  ; ASCII-Code im Bildschirmspeicher ablegen

WSCR    call  LCD_GetSCRADR  ; Adresse im Bildschirmspeicher berechnen
    movwf  FSR
    movf  ASCIICode,w
    movwf  0    ; aktuelles Zeichen abspeichern

  ; Kursor auf nchstes Zeichen setzen

    incf  XPosC,f
    movlw  14
    subwf  XPosC,w
    btfss  STATUS,Z  ; letztes Zeichen in Zeile?
    goto  CharDec

    clrf  XPosC
    incf  YPos,f    ; Zeile erhhen

    movlw  6
    subwf  YPos,w
    btfss  STATUS,Z  ; letzte Zeile auf Display?
    goto  CharDec

    clrf  YPos    ; wieder auf x=0, y=0

CharDec    incf  MerkFSR,w
    movwf  FSR    ; FSR+1 wiederherstellen (auf nchstes darzustellendes Zeichen)
    decfsz  BZ3,f    ; alle Zeichen dargestellt?
    goto  NextChar

    return

;  ****  Zeichensatz-Bitmuster aus Tabelle lesen    ****
;  ****  ASCII-Code = zu lesendes zeichen    ****
;  ****  ASCIIOffset = Offset aktuelles Bitmuster  ****

GetASCIIByte  clrf  ADRH
    movlw  (EndCodeTable-CodeTable) / 5  ; Anzahl gespeicherter Zeichen
    subwf  ASCIICode,w
    btfsc  STATUS,C
    retlw  0    ; Zeichen nicht definiert, als Lehrzeichen darstellen

  ; Adresse in Tabelle Berechnen
  ; ADR=CodeTable+(5*ASCIICode)+ASCIIOffset

    movf  ASCIICode,w
    movwf  ADRL

    bcf  STATUS,C
    rlf  ADRL,f    ; *2
    rlf  ADRH,F    ; evtl. bertrag
          ; in ADRH/ADRL jetzt 2*ASCIICode
    rlf  ADRL,f    ; *2
    rlf  ADRH,F    ; evtl. bertrag
          ; in ADRL jetzt 4*ASCIICode
    addwf  ADRL,f    ; +w (ASCIICode)

    btfsc  STATUS,C  ; bertrag?
    incf  ADRH,f    ; verrechnen
          ; in ADRL jetzt 5*ASCIICode

    movf  ASCIIOffset,w  ; Offset fr aktuelles Bitmuster
    addwf  ADRL,f
    btfsc  STATUS,C
    incf  ADRH,f

  ; Adresse der TAbelle noch dazuaddieren

    movlw  LOW(CodeTable)
    addwf  ADRL,f
    btfsc  STATUS,C
    incf  ADRH,f

    movlw  HIGH(CodeTable)
    addwf  ADRH,w

    movwf  PCLATH
    movf  ADRL,w
    movwf  PCL    ; Sprung (Datenbyte lesen)

CodeTable
  ; Zeichensatzdefinition
  ; Zeichen 0..9F
  ; 795 Byte Programmspeicher

  DT 0x5C,0x62,0x02,0x62,0x5C    ; ASCII-Code 0x00 (0)
  DT 0x12,0x19,0x15,0x12,0x00    ; ASCII-Code 0x01 (1)
  DT 0x11,0x15,0x15,0x0A,0x00    ; ASCII-Code 0x02 (2)
  DT 0x7F,0x00,0x00,0x00,0x00    ; ASCII-Code 0x03 (3)
  DT 0x7F,0x7F,0x00,0x00,0x00    ; ASCII-Code 0x04 (4)
  DT 0x7F,0x7F,0x7F,0x00,0x00    ; ASCII-Code 0x05 (5)
  DT 0x7F,0x7F,0x7F,0x7F,0x00    ; ASCII-Code 0x06 (6)
  DT 0x7F,0x7F,0x7F,0x7F,0x7F    ; ASCII-Code 0x07 (7)
  DT 0x80,0x80,0x80,0x80,0x80    ; ASCII-Code 0x08 (8)
  DT 0xC0,0xC0,0xC0,0xC0,0xC0    ; ASCII-Code 0x09 (9)
  DT 0xE0,0xE0,0xE0,0xE0,0xE0    ; ASCII-Code 0x0A (10)
  DT 0xF0,0xF0,0xF0,0xF0,0xF0    ; ASCII-Code 0x0B (11)
  DT 0xF8,0xF8,0xF8,0xF8,0xF8    ; ASCII-Code 0x0C (12)
  DT 0xFC,0xFC,0xFC,0xFC,0xFC    ; ASCII-Code 0x0D (13)
  DT 0xFE,0xFE,0xFE,0xFE,0xFE    ; ASCII-Code 0x0E (14)
  DT 0xFF,0xFF,0xFF,0xFF,0xFF    ; ASCII-Code 0x0F (15)
  DT 0x00,0x7F,0x3E,0x1C,0x08    ; ASCII-Code 0x10 (16)
  DT 0x08,0x1C,0x3E,0x7F,0x00    ; ASCII-Code 0x11 (17)
  DT 0x06,0x05,0x00,0x06,0x05    ; ASCII-Code 0x12 (18)
  DT 0x05,0x03,0x00,0x05,0x03    ; ASCII-Code 0x13 (19)
  DT 0x44,0x66,0x77,0x66,0x44    ; ASCII-Code 0x14 (20)
  DT 0x11,0x33,0x77,0x33,0x11    ; ASCII-Code 0x15 (21)
  DT 0x1C,0x3E,0x3E,0x3E,0x1C    ; ASCII-Code 0x16 (22)
  DT 0x10,0x38,0x54,0x10,0x1F    ; ASCII-Code 0x17 (23)
  DT 0x04,0x02,0x7F,0x02,0x04    ; ASCII-Code 0x18 (24)
  DT 0x10,0x20,0x7F,0x20,0x10    ; ASCII-Code 0x19 (25)
  DT 0x08,0x08,0x2A,0x1C,0x08    ; ASCII-Code 0x1A (26)
  DT 0x08,0x1C,0x2A,0x08,0x08    ; ASCII-Code 0x1B (27)
  DT 0x40,0x44,0x4A,0x51,0x40    ; ASCII-Code 0x1C (28)
  DT 0x40,0x51,0x4A,0x44,0x40    ; ASCII-Code 0x1D (29)
  DT 0x20,0x38,0x3E,0x38,0x20    ; ASCII-Code 0x1E (30)
  DT 0x02,0x0E,0x3E,0x0E,0x02    ; ASCII-Code 0x1F (31)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x20 (32)
  DT 0x00,0x00,0x4F,0x00,0x00    ; ASCII-Code 0x21 (33)
  DT 0x00,0x07,0x00,0x07,0x00    ; ASCII-Code 0x22 (34)
  DT 0x14,0x7F,0x14,0x7F,0x14    ; ASCII-Code 0x23 (35)
  DT 0x24,0x2A,0x7F,0x2A,0x12    ; ASCII-Code 0x24 (36)
  DT 0x23,0x13,0x08,0x64,0x62    ; ASCII-Code 0x25 (37)
  DT 0x36,0x49,0x55,0x22,0x50    ; ASCII-Code 0x26 (38)
  DT 0x00,0x05,0x03,0x00,0x00    ; ASCII-Code 0x27 (39)
  DT 0x00,0x1C,0x22,0x41,0x00    ; ASCII-Code 0x28 (40)
  DT 0x00,0x41,0x22,0x1C,0x00    ; ASCII-Code 0x29 (41)
  DT 0x14,0x08,0x3E,0x08,0x14    ; ASCII-Code 0x2A (42)
  DT 0x08,0x08,0x3E,0x08,0x08    ; ASCII-Code 0x2B (43)
  DT 0x00,0x50,0x30,0x00,0x00    ; ASCII-Code 0x2C (44)
  DT 0x08,0x08,0x08,0x08,0x08    ; ASCII-Code 0x2D (45)
  DT 0x00,0x60,0x60,0x00,0x00    ; ASCII-Code 0x2E (46)
  DT 0x20,0x10,0x08,0x04,0x02    ; ASCII-Code 0x2F (47)
  DT 0x3E,0x51,0x49,0x45,0x3E    ; ASCII-Code 0x30 (48)
  DT 0x00,0x42,0x7F,0x40,0x00    ; ASCII-Code 0x31 (49)
  DT 0x42,0x61,0x51,0x49,0x46    ; ASCII-Code 0x32 (50)
  DT 0x21,0x41,0x45,0x4B,0x31    ; ASCII-Code 0x33 (51)
  DT 0x18,0x14,0x12,0x7F,0x10    ; ASCII-Code 0x34 (52)
  DT 0x27,0x45,0x45,0x45,0x39    ; ASCII-Code 0x35 (53)
  DT 0x3C,0x4A,0x49,0x49,0x30    ; ASCII-Code 0x36 (54)
  DT 0x01,0x01,0x79,0x05,0x03    ; ASCII-Code 0x37 (55)
  DT 0x36,0x49,0x49,0x49,0x36    ; ASCII-Code 0x38 (56)
  DT 0x06,0x49,0x49,0x29,0x1E    ; ASCII-Code 0x39 (57)
  DT 0x00,0x36,0x36,0x00,0x00    ; ASCII-Code 0x3A (58)
  DT 0x00,0x56,0x36,0x00,0x00    ; ASCII-Code 0x3B (59)
  DT 0x08,0x14,0x22,0x41,0x00    ; ASCII-Code 0x3C (60)
  DT 0x14,0x14,0x14,0x14,0x14    ; ASCII-Code 0x3D (61)
  DT 0x00,0x41,0x22,0x14,0x08    ; ASCII-Code 0x3E (62)
  DT 0x02,0x01,0x51,0x09,0x06    ; ASCII-Code 0x3F (63)
  DT 0x32,0x79,0x79,0x41,0x3E    ; ASCII-Code 0x40 (64)
  DT 0x7E,0x11,0x11,0x11,0x7E    ; ASCII-Code 0x41 (65)
  DT 0x41,0x7F,0x49,0x49,0x36    ; ASCII-Code 0x42 (66)
  DT 0x3E,0x41,0x41,0x41,0x22    ; ASCII-Code 0x43 (67)
  DT 0x41,0x7F,0x41,0x41,0x3E    ; ASCII-Code 0x44 (68)
  DT 0x7F,0x49,0x49,0x49,0x49    ; ASCII-Code 0x45 (69)
  DT 0x7F,0x09,0x09,0x09,0x01    ; ASCII-Code 0x46 (70)
  DT 0x3E,0x41,0x41,0x49,0x7A    ; ASCII-Code 0x47 (71)
  DT 0x7F,0x08,0x08,0x08,0x7F    ; ASCII-Code 0x48 (72)
  DT 0x00,0x41,0x7F,0x41,0x00    ; ASCII-Code 0x49 (73)
  DT 0x20,0x40,0x41,0x3F,0x01    ; ASCII-Code 0x4A (74)
  DT 0x7F,0x08,0x14,0x22,0x41    ; ASCII-Code 0x4B (75)
  DT 0x7F,0x40,0x40,0x40,0x40    ; ASCII-Code 0x4C (76)
  DT 0x7F,0x02,0x06,0x02,0x7F    ; ASCII-Code 0x4D (77)
  DT 0x7F,0x06,0x08,0x30,0x7F    ; ASCII-Code 0x4E (78)
  DT 0x3E,0x41,0x41,0x41,0x3E    ; ASCII-Code 0x4F (79)
  DT 0x7F,0x09,0x09,0x09,0x06    ; ASCII-Code 0x50 (80)
  DT 0x3E,0x41,0x51,0x21,0x5E    ; ASCII-Code 0x51 (81)
  DT 0x7F,0x09,0x19,0x29,0x46    ; ASCII-Code 0x52 (82)
  DT 0x26,0x49,0x49,0x49,0x32    ; ASCII-Code 0x53 (83)
  DT 0x01,0x01,0x7F,0x01,0x01    ; ASCII-Code 0x54 (84)
  DT 0x3F,0x40,0x40,0x40,0x3F    ; ASCII-Code 0x55 (85)
  DT 0x1F,0x20,0x40,0x20,0x1F    ; ASCII-Code 0x56 (86)
  DT 0x7F,0x20,0x18,0x20,0x7F    ; ASCII-Code 0x57 (87)
  DT 0x63,0x14,0x08,0x14,0x63    ; ASCII-Code 0x58 (88)
  DT 0x07,0x08,0x70,0x08,0x07    ; ASCII-Code 0x59 (89)
  DT 0x61,0x51,0x49,0x45,0x43    ; ASCII-Code 0x5A (90)
  DT 0x00,0x7F,0x41,0x41,0x00    ; ASCII-Code 0x5B (91)
  DT 0x02,0x04,0x08,0x10,0x20    ; ASCII-Code 0x5C (92)
  DT 0x00,0x41,0x41,0x7F,0x00    ; ASCII-Code 0x5D (93)
  DT 0x04,0x02,0x01,0x02,0x04    ; ASCII-Code 0x5E (94)
  DT 0x40,0x40,0x40,0x40,0x40    ; ASCII-Code 0x5F (95)
  DT 0x01,0x02,0x04,0x00,0x00    ; ASCII-Code 0x60 (96)
  DT 0x20,0x54,0x54,0x54,0x78    ; ASCII-Code 0x61 (97)
  DT 0x7F,0x48,0x44,0x44,0x38    ; ASCII-Code 0x62 (98)
  DT 0x38,0x44,0x44,0x44,0x28    ; ASCII-Code 0x63 (99)
  DT 0x38,0x44,0x44,0x48,0x7F    ; ASCII-Code 0x64 (100)
  DT 0x38,0x54,0x54,0x54,0x18    ; ASCII-Code 0x65 (101)
  DT 0x00,0x08,0x7E,0x09,0x02    ; ASCII-Code 0x66 (102)
  DT 0x18,0xA4,0xA4,0x98,0x7C    ; ASCII-Code 0x67 (103)
  DT 0x7F,0x08,0x04,0x04,0x78    ; ASCII-Code 0x68 (104)
  DT 0x00,0x44,0x7D,0x40,0x00    ; ASCII-Code 0x69 (105)
  DT 0x40,0x80,0x84,0x7D,0x00    ; ASCII-Code 0x6A (106)
  DT 0x00,0x7F,0x10,0x28,0x44    ; ASCII-Code 0x6B (107)
  DT 0x00,0x41,0x7F,0x40,0x00    ; ASCII-Code 0x6C (108)
  DT 0x7C,0x04,0x78,0x04,0x78    ; ASCII-Code 0x6D (109)
  DT 0x7C,0x08,0x04,0x04,0x78    ; ASCII-Code 0x6E (110)
  DT 0x38,0x44,0x44,0x44,0x38    ; ASCII-Code 0x6F (111)
  DT 0xFC,0x18,0x24,0x24,0x18    ; ASCII-Code 0x70 (112)
  DT 0x18,0x24,0x24,0x18,0xFC    ; ASCII-Code 0x71 (113)
  DT 0x7C,0x08,0x04,0x04,0x08    ; ASCII-Code 0x72 (114)
  DT 0x58,0x54,0x54,0x54,0x64    ; ASCII-Code 0x73 (115)
  DT 0x04,0x3F,0x44,0x40,0x20    ; ASCII-Code 0x74 (116)
  DT 0x3C,0x40,0x40,0x3C,0x40    ; ASCII-Code 0x75 (117)
  DT 0x1C,0x20,0x40,0x20,0x1C    ; ASCII-Code 0x76 (118)
  DT 0x3C,0x40,0x30,0x40,0x3C    ; ASCII-Code 0x77 (119)
  DT 0x44,0x28,0x10,0x28,0x44    ; ASCII-Code 0x78 (120)
  DT 0x1C,0xA0,0xA0,0x90,0x7C    ; ASCII-Code 0x79 (121)
  DT 0x44,0x64,0x54,0x4C,0x44    ; ASCII-Code 0x7A (122)
  DT 0x00,0x08,0x36,0x41,0x00    ; ASCII-Code 0x7B (123)
  DT 0x00,0x00,0x7F,0x00,0x00    ; ASCII-Code 0x7C (124)
  DT 0x00,0x41,0x36,0x08,0x00    ; ASCII-Code 0x7D (125)
  DT 0x10,0x08,0x08,0x10,0x08    ; ASCII-Code 0x7E (126)
  DT 0x00,0x06,0x09,0x09,0x06    ; ASCII-Code 0x7F (127)
  DT 0x30,0x28,0x24,0x28,0x30    ; ASCII-Code 0x80 (128)
  DT 0x38,0x42,0x40,0x3A,0x40    ; ASCII-Code 0x81 (129)
  DT 0x03,0x03,0x38,0x44,0x44    ; ASCII-Code 0x82 (130)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x83 (131)
  DT 0x20,0x55,0x54,0x55,0x78    ; ASCII-Code 0x84 (132)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x85 (133)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x86 (134)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x87 (135)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x88 (136)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x89 (137)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x8A (138)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x8B (139)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x8C (140)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x8D (141)
  DT 0x78,0x25,0x24,0x25,0x78    ; ASCII-Code 0x8E (142)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x8F (143)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x90 (144)
  DT 0x24,0x54,0x7C,0x54,0x58    ; ASCII-Code 0x91 (145)
  DT 0x7E,0x11,0x7F,0x49,0x41    ; ASCII-Code 0x92 (146)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x93 (147)
  DT 0x30,0x4A,0x48,0x4A,0x30    ; ASCII-Code 0x94 (148)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x95 (149)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x96 (150)
  DT 0x00,0x00,0x00,0x00,0x00    ; ASCII-Code 0x97 (151)
  DT 0x1C,0x36,0x55,0x55,0x41    ; ASCII-Code 0x98 (152)
  DT 0x38,0x45,0x44,0x45,0x38    ; ASCII-Code 0x99 (153)
  DT 0x38,0x42,0x40,0x42,0x38    ; ASCII-Code 0x9A (154)
  DT 0x1C,0x22,0x7F,0x22,0x14    ; ASCII-Code 0x9B (155)
  DT 0x68,0x7E,0x29,0x41,0x22    ; ASCII-Code 0x9C (156)
  DT 0x29,0x2A,0x7C,0x2A,0x29    ; ASCII-Code 0x9D (157)
  DT 0x77,0x05,0x12,0x7C,0x50    ; ASCII-Code 0x9E (158)
  DT 0x48,0x7E,0x49,0x01,0x02    ; ASCII-Code 0x9F (159)
EndCodeTable
