;   Datensatzaufbau
; ===================
;
; Als erstes Byte wird die Anzahl noch folgender Datensatzbytes gesendet.
;-        .
; Dadurch ist die schnelle Erkennung des Endes des Datensatzes ohne Timeout
; oder spezielle Sonderzeichen mglich.
;-        - 
;-  .
; Danach folgt das Befehlsbyte, welches den Befehl eindeutig kennzeichnet
; und die eigentlichen Nutzdaten falls fr den Befehl notwendig.
;-    ,     
;-  ,     .
; Als letztes Byte wird als Checksumme (BCC) die XOR-Verknpfung aller
; Datensatzbyte gesendet.
;-      XOR,  
;-        (BCC).
; Die maximale Datensatzlnge betrgt 19 Byte!
;-    - 19 !

;   Antworttelegramm    
; ==================== ====================
;  ()  (,)    .
;       () ()
; (Interupt ).
;  Datenuebetragung Timout ,     Gesendet.
;     ,      
; NAK gesendet .
; ====================
; Jeder Befehl mit richtiger Lnge wird nach Abarbeitung beantwortet. Vor Beantwortung
; werden keine weiteren Befehle bearbeitet (Interupt ist gesperrt).
; Wenn die Datenbetragung durch ein Timout unterbrochen wird, wird keine Antwort
; gesendet. Grund dafr ist, das nicht bei jeder Strung auf der Empfangsleitung ein
; NAK gesendet wird.

; Der Aufbau des Telegrammes ist mit dem Befehlsdatensatz identisch, nur das
; anstatt des Befehlsbytes ein Statusbyte gesendet wird.
; ()      , 
;  ,  Befehlsbytes Statusbyte gesendet .

; Das Statusbyte hat folgende Bedeutung:
; Statusbyte   :
; 0x00  --> Befehlsausfhrung OK (ACK)   OK
; 0xFF  --> Fehler im Befehlsdatensatz (NAK)    
; 0xF2  --> Befehl nicht untersttzt (CNS) ()  

;  ****   ()  ser.  
;  ****  Auswertung der Befehle von ser. Schnittstelle
;  ****     ,   ISR
;  ****  Wird vom Hauptprogramm aufgerufen, wenn in ISR
;  ****     (n + 1 Byte  n = 1. Byte)
;  **** 
;  ****  das Ende eines gltigen Datensatzes (n+1 Byte empfangen n= 1. Byte)
;  ****  erkannt wurde.

BefehlAusw  bcf  INTCON,GIE  ; whrend Bearbeitung Interupts gesperrt
    bcf  _DataRecived

  ; Checksumme testen

    movlw  ComBuff
    movwf  FSR

    decf  Z_Empfang,w  ; BCC ber alle empfangene Zeichen
    movwf  BZ1
    movlw  0

BNextCheck  xorwf  0,w    ; BCC=XOR aller Datenbytes
    incf  FSR,f    ; auf nchstes Datenbyte
    decfsz  BZ1,f
    goto  BNextCheck

    subwf  0,w
    btfss  STATUS,Z
    goto  BSendNAK  ; Checksumme falsch

  ; Daten OK

    movf  ComBuff+1,w  ; auf Befehlsbyte
    movwf  FSR
    clrf  BZ1

NextBefSequ  call  BefehlSequ  ; Befehl in Tabelle suchen
    iorlw  0
    btfsc  STATUS,Z  ; Ende Befehlstabelle ?
    goto  BSendCNS  ; Befehl nicht untersttzt

    subwf  ComBuff+1,w  ; Befehl gefunden ?
    btfsc  STATUS,Z
    goto  Befehl    ; ja, dann ausfhren

    incf  BZ1,f    ; nchster Befehl in Tabelle
    goto  NextBefSequ

; Befehlstabelle   

BefehlSequ  movlw  LOW(BefehlsByte)  ; Low-Byte Adresse Tabellenanfang
    movwf  ADRL
    movlw  HIGH(BefehlsByte)  ; High-Byte Adresse Tabellenanfang

    goto  ToPCLath
Befehl    movlw  LOW(BefehlsAktion)  ; Low-Byte Adresse Tabellenanfang
    movwf  ADRL
    movlw  HIGH(BefehlsAktion)  ; High-Byte Adresse Tabellenanfang

ToPCLath  movwf  PCLATH      ; High-Byte in PCLATH
    bcf  STATUS,C
    rlf  BZ1,w    ; 2*BZ1 (Zhler) -->w
    addwf  ADRL,w    ; Speicheradresse bilden
    btfsc  STATUS,C  ; bertrag?
    incf  PCLATH,f
    movwf  PCL    ; Sprung

BefehlsByte
    retlw  'R'
BefehlsAktion
    goto  BResetSystem  ; ResetSystem
          ; Daten vom Host 02, 'R', BCC
    retlw  'S'
    goto  BSetTCursor  ; SetTextCursor
          ; Daten vom Host 04, 'S', XPos, YPos, BCC
    retlw  's'
    goto  BSetGCursor  ; SetGrafikCursor
          ; Daten vom Host 04, 's', XPos, YPos, BCC
    retlw  'C'
    goto    BCLS    ; CLS
          ; Daten vom Host 02, 'C', BCC
    retlw  'P'
    goto  BPrintString  ; PrintString
          ; Daten vom Host n+3, 'P', Chr0..Chrn, BCC !!!n<=15
    retlw  'H'
    goto  BPrintHexData  ; PrintHexData (Grafikdaten)
          ; Daten vom Host n+3, 'H', Byte0..Byten, BCC !!!n<=15
    retlw  'M'
    goto  BSetLCDMode  ; SetLCDMode
          ; Daten vom Host 03, 'M', ModeByte, BCC
          ; ModeByte=0 --> LCD im Normalmode
          ; Modebyte<>0 --> LCD invers
    retlw  'K'
    goto  BSetCursorMode  ; SetCursorMode
          ; Daten vom Host 03, 'K', CModeByte, BCC
          ; CModeByte ist Maske fr XOR-Verknpfung mit Zeiche unter Kursor
          ; CModeByte=0 --> Kursor unsichtbar
          ; CModeByte=1 --> Zeichen blinkend unterstrichen
          ; CModeByte=0xFF --> Zeichen blinkt invers
    retlw  'T'
    goto  BGetKeyBuffer  ; GetKeyBuffer
          ; Daten vom Host 02, 'T', BCC
  ; Ende Befehlstabelle
    retlw  0    ; Befehl falsch



;  ****  Befehlsimplementation  ****

  ; reset System
  ; Daten vom Host 02, 'R', BCCBResetSystem
BResetSystem  call  InitPIC
    call  InitDisplay
    goto  BSendACK

  ; SetTextCursor
  ; Daten vom Host 04, 'S', XPos, YPos, BCC
BSetTCursor  btfss  _CursorOn  ; Kursor aktuell invertiert?
    goto  BSC1
;   ja, Zeichen an Kursorposition richtig darstellen

    bcf  _CursorOn
    bsf  _ShowCursor
;  Zeichen an Kursorposition aus Bildschirmspeicher lesen

    call  LCD_GetSCRADR    ; Adresse im Bildschirmspeicher ermitteln
    movwf  FSR      ; in FSR

    movlw  1
    call  LCD_PrintStr    ; aktuelles Zeichen "normal" darstellen
    bcf  _ShowCursor

BSC1    movf  ComBuff+2,w    ; XPos aus Emfangspuffer
    movwf  XPosC
    movf  ComBuff+3,w    ; YPos aus Empangspuffer
    movwf  YPos
    call  LCD_SetCursor    ; Kursor (Adresse im LCD-Speicher) setzen
    goto  BSendACK

  ; SetGrafikCursor
  ; Daten vom Host 04, 's', XPos, YPos, BCC
BSetGCursor  movf  ComBuff+2,w
    iorlw  10000000b  ; LCD-Command set X adr bilden
    movwf  DispBuff

    movf  ComBuff+3,w
    iorlw  01000000b  ; LCD-Command set y adr bilden
    movwf  DispBuff+1

    movlw  2    ; 2Byte zu bertragen
    call  LCD_ComSend
    goto  BSendACK


  ; CLS
  ; Daten vom Host 02, 'C', BCC
BCLS    call  InitDisplay  ; Display komplett initialisieren
    goto  BSendACK

  ; PrintString
  ; Daten vom Host n+3, 'P', Chr0..Chrn, BCC
BPrintString  movlw  ComBuff+2    ; Begin des Strings
    movwf  FSR
    movlw  2
    subwf  ComBuff,w    ; Anzahl Zeichen = Anzahl DatensatzZeichen-2
    bcf  _CursorOn  ; sonst werden Zeichen evtl. invertiert
    call  LCD_PrintStr  ; Ausgabe
    goto  BSendACK

  ; PrintHexData (Grafikdaten)
  ; Daten vom Host n+3, 'H', Byte0..Byten, BCC
BPrintHexData  movlw  ComBuff+2
    movwf  FSR    ; Begin der Grafikdaten
    movlw  2
    subwf  ComBuff,w    ; Anzahl Byte = Anzahl Byte-2
    call  LCD_GrafikData  ; Ausgabe
    goto  BSendACK


  ; SetLCDMode
  ; Daten vom Host 03, 'M', ModeByte, BCC
  ; ModeByte=0 --> LCD im Normalmode
  ; Modebyte<>0 --> LCD invers
BSetLCDMode  movf  ComBuff+2,w
    btfss  STATUS,Z
    goto  InvertLCD

    movlw  00001100b  ; Normal-Mode
    movwf  DispBuff
    movlw  1    ; 1 Byte zu senden
    call  LCD_ComSend
    goto  BSendACK
InvertLCD  movlw  00001101b  ; Invers-Mode
    movwf  DispBuff
    movlw  1    ; 1 Byte zu senden
    call  LCD_ComSend
    goto  BSendACK

  ; SetCursorMode
  ; Daten vom Host 03, 'K', CModeByte, BCC
  ; CModeByte ist Maske fr XOR-Verknpfung mit Zeiche unter Kursor
  ; CModeByte=0 --> Kursor unsichtbar
  ; CModeByte=8 --> Zeichen blinkend unterstrichen
  ; CModeByte=0xFF --> Zeichen blinkt invers
BSetCursorMode  movf  ComBuff+2,w
    movwf  CursorMask
    btfss  STATUS,Z
    goto  BSendACK

    btfss  _CursorOn    ; Kursor aktuell sichtbar?
    goto  BSendACK
;   ja, Zeichen richtig darstellen

    bcf  _CursorOn
    bsf  _ShowCursor
;  Zeichen an Kursorposition aus Bildschirmspeicher lesen

    call  LCD_GetSCRADR    ; Adresse ermitteln
    movwf  FSR      ; in FSR

    movlw  1
    call  LCD_PrintStr    ; aktuelles Zeichen darstellen

    movf  CursorMask,w
    btfss  STATUS,Z    ; Wenn <>0 Cursor anzeigen
    bsf  _ShowCursor

    goto  BSendACK

  ; GetKeyBuffer
  ; Daten vom Host 02, 'T', XPos, YPos, BCC
BGetKeyBuffer  bsf  _INT    ; Interuptausgang zurcksetzen
    movlw  7
    movwf  SBuff

  ; Tastaturpuffer in Sendepuffer
    clrf  SBuff+1    ; Statusbyte
    movf  KeyBuff,w
    movwf  SBuff+2
    movf  KeyBuff+1,w
    movwf  SBuff+3
    movf  KeyBuff+2,w
    movwf  SBuff+4
    movf  KeyBuff+3,w
    movwf  SBuff+5
    movf  KeyBuff+4,w
    movwf  SBuff+6

  ; Tastaturpuffer lschen

    clrf  KeyBuff
    clrf  KeyBuff+1
    clrf  KeyBuff+2
    clrf  KeyBuff+3
    clrf  KeyBuff+4

    goto  BSendData

;   ****  Statusantworten

BSendNAK  movlw  0xff    ; NAK
    goto  BSendState

BSendCNS  movlw  0xf2    ; Befehl nicht untersttzt
    goto  BSendState

BSendACK  movlw  0    ; ACK

BSendState  movwf  SBuff+1    ; Sttusbyte in Sendepuffer
    movlw  2        ;   
    movwf  SBuff    ; Lnge Statusantwort


;  ****  zu sendende Daten aufbereiten und bertragen

BSendData  movf  SBuff,w
    movwf  BZ1    ; Anzahl zu sendender Bytes-1
                  ;   Bytes-1
  ; Checksumme (BCC) berechnen
    clrwdt
    movlw  SBuff
    movwf  FSR
    movlw  0

NextBCC    xorwf  0,w
    incf  FSR,f
    decfsz  BZ1,f
    goto  NextBCC

    movwf  0    ; BCC in Sendepuffer

    goto  SendCommData  ; Daten absenden