;  ********************************************
;  ********************************************
;  ****          ****
;  ****  UART-Routinen fr PIC16F682X  ****
;  ****          ****
;  ********************************************
;  ********************************************

;  ****  Empf_COM            ****
;       ****    Empfang serieller Daten (wird von ISR aufgerufen)       ****

Empf_COM
    movlw  100
    movwf  TMR0       ; TMR0 neu initialisieren
    movlw  3
    movwf  UARTTimer  ; 30ms Timeout

    btfss  RCSTA,FERR ; Bit 2 Frame-Error (kein Stoppbit)
    goto   Read_Byte
    movf   RCREG,w    ; "falsches" Byte auslesen und zurck (nicht in Empfangspuffer)
    return

Read_Byte
    bsf    _Empfang   ; Empfang luft
InBuff
    call   CheckParity; Paritt testen
    btfss  Paritaet,0 ; Paritt OK ?
    goto   RCStore
    return            ; Fehler

RCStore
    movf   P_EBuff,w
    movwf  FSR        ; FSR auf aktuelles Byte im Empfangspuffer

    movf   LastByte,w ; empfangenes Byte (steht nach Parittstest in LastByte)
                      ; in Empfangsbuffer
    movwf  0

  ; Test ob Datensatz zu Ende
    movf   Z_Empfang,w
    subwf  ComBuff,w   ; Empfangene Bytezahl-1 mit erstem Byte identisch ?
    btfss  STATUS,Z
    goto   E_COMInc    ; nein
    incf   Z_Empfang,f ; +1
    bsf    _DataRecived; ja --> Ende Datenbertragung signalisieren
    bcf    _Empfang    ; alle Daten empfangen
    return

  ;  nchste Position im Empfangspuffer
E_COMInc
    incf    P_EBuff,f    ; Pufferposition erhhen
    incf    Z_Empfang,f  ; Zhler erhhen
    movlw   ComBuff+0x14 ; Ende ebuff
    subwf   P_EBuff,w    ; Sprung, wenn grer
    btfss   STATUS,Z
    return

  ; Empfangspufferberlauf
    movlw   ComBuff
    movwf   P_EBuff       ; wieder auf Anfang
End_Ri
    return


;  ****    Send_COM            ****
;  ****    Senden sendet Daten, wenn (wird von ISR aufgerufen)  ****

Send_COM
    bcf     STATUS,RP0
    incf    P_SBuff,f
    decfsz  Z_Send,f  ; letztes Byte gesendet ?
    goto    NByte_Send

;  warten bis letztes Byte abgesendet

Buff_Empty
    bsf    STATUS,RP0
    btfss  TXSTA,TRMT  ; Byte abgesendet ?
    goto   Buff_Empty
    bsf    STATUS,RP0
    bcf    PIE1,TXIE  ; Sendeinterupt sperren
    bcf    TXSTA,TXEN  ; Senden sperren
    bsf    PIE1,RCIE  ; Empfangsinterupt freigeben
    bsf    INTCON,T0IE  ; RTCC-interupt freigeben
    bcf    STATUS,RP0
    bsf    RCSTA,CREN
    movlw  ComBuff
    movwf  P_EBuff    ; Zeiger Empfangspuffer wieder auf Anfang
    clrf   Z_Empfang
    return

  ; nchstes Byte aus Sendepuffer senden
NByte_Send
    movf   P_SBuff,w
    movwf  FSR
    call   GetParity
    movf   0,w
    movwf  TXREG     ; nchstes Byte senden
    return

;  ****  SendCommData Initialisiert die Datenbertragung ber die serielle Schnittstelle  ****
;  ****  zu sendende Daten stehen ab SBuff im Ram          ****
;  ****  Byte0 ist die Anzahl zu bertragender Bytes-1     ****

SendCommData
    incf   SBuff,w
    movwf  Z_Send      ; Anzahl zu sendender Bytes -   Bytes
    movlw  SBuff
    movwf  P_SBuff     ; Zeiger auf Anfang Sendepuffer -    Buff 
    movwf  FSR

    call   GetParity

    movf   INDF,w      ; Byte  Buff
    movwf  TXREG       ; erstes Byte in Senderegister - Byte  Senderegister
                       ; Rest wird von ISR erledigt
    bcf    RCSTA,CREN  ; Empfang sperren -  ()

    bsf    STATUS,RP0
    bsf    TXSTA,TXEN  ; Senden freigeben -  
    bsf    PIE1,TXIE   ; Sendeinterupt freigeben -  
    bcf    STATUS,RP0

    bsf    INTCON,GIE  ; Interupts freigeben -  
    return

;  ****  Parittsberechnung
;  ****  knnte man zwar auch per Schleife machen aber so gehts schneller

CheckParity
    movf    RCREG,w    ;  Read the character from RCREG
    movwf   LastByte   ;  Save it into LastByte REG

    clrf    Paritaet
    btfsc   LastByte,0
    incf    Paritaet,f
    btfsc   LastByte,1
    incf    Paritaet,f
    btfsc   LastByte,2
    incf    Paritaet,f
    btfsc   LastByte,3
    incf    Paritaet,f
    btfsc   LastByte,4
    incf    Paritaet,f
    btfsc   LastByte,5
    incf    Paritaet,f
    btfsc   LastByte,6
    incf    Paritaet,f
    btfsc   LastByte,7
    incf    Paritaet,f

    movf    RCSTA,w
    xorwf   Paritaet,f  ; Vergleich (nur Bit 0 interessant)
    return

GetParity
    clrf    Paritaet
    btfsc   INDF,0
    incf    Paritaet,f
    btfsc   INDF,1
    incf    Paritaet,f
    btfsc   INDF,2
    incf    Paritaet,f
    btfsc   INDF,3
    incf    Paritaet,f
    btfsc   INDF,4
    incf    Paritaet,f
    btfsc   INDF,5
    incf    Paritaet,f
    btfsc   INDF,6
    incf    Paritaet,f
    btfsc   INDF,7
    incf    Paritaet,f

    movlw   00000001b
    andwf   Paritaet,w

    bsf     STATUS,RP0
    bcf     TXSTA,0
    iorwf   TXSTA,f    ;  0: TX9D: 9-    - 
    bcf     STATUS,RP0

    return